@@ -4,7 +4,7 @@
 __author__ = 'Frederic Escudie'
 __copyright__ = 'Copyright (C) 2017 IUCT-O'
 __license__ = 'GNU General Public License'
-__version__ = '1.18.0'
+__version__ = '1.21.0'
 __email__ = 'escudie.frederic@iuct-oncopole.fr'
 __status__ = 'prod'
 
@@ -49,6 +49,9 @@ class SampleSheetIO(object):
         self.header = self._getInfoFromSection(sections_by_title["Header"])
         self.manifests = self._getInfoFromSection(sections_by_title["Manifests"]) if "Manifests" in sections_by_title else dict()
         self.run = self._getRunFromHeaderAndReads(sections_by_title["Header"], sections_by_title["Reads"])
+        # Post process
+        if "Description" not in self.header:
+            self.header["Description"] = ""
 
     def _getSamplesFromData(self, data_section):
         samples = list()
@@ -57,6 +60,8 @@ class SampleSheetIO(object):
             spl = {data_titles[idx]: field.strip() for idx, field in enumerate(line.split(","))}
             spl["Sample_Basename"] = getIlluminaName(spl["Sample_ID"])
             spl["Library_Basename"] = spl["Sample_Basename"] + "_S" + str(spl_idx + 1)
+            if "Sample_Description" in spl:
+                spl["Description"] = spl["Sample_Description"]
             samples.append(spl)
         return samples
 
@@ -139,7 +144,7 @@ class ADSSampleSheetIO(SampleSheetIO):
         :type end_pattern: str
         :param subject: "library" or "sample" to select files corresponding to libraries or corresponding to samples.
         :type subject: str
-        :return: By subject the pathes of the corresponding files.
+        :return: By subject the paths of the corresponding files.
         :rtype: dict
         """
         subject_start_tag = subject.capitalize()
@@ -155,7 +160,7 @@ class ADSSampleSheetIO(SampleSheetIO):
         """
         Add the files in directory corresponding to the subject. These information are keyed by provided tag for each element of self.samples.
 
-        :param tag: The attribute used in self.sample to store the files pathes.
+        :param tag: The attribute used in self.sample to store the files paths.
         :type tag: str
         :param directory: The path to the directory where the files names are evaluated.
         :type directory: str
@@ -170,6 +175,54 @@ class ADSSampleSheetIO(SampleSheetIO):
             spl[tag] = files_by_spl[spl[subject_tag]]
 
 
+class Bcl2fastqLog(object):
+    """Reader for bcl2fastq log."""
+
+    def __init__(self, path):
+        self.command = None
+        self.complete = None
+        self.end_time = None
+        self.filepath = path
+        self.start_time = None
+        self.version = None
+        self._parse()
+
+    def _parse(self):
+        self.command = None
+        self.complete = None
+        self.end_time = None
+        self.start_time = None
+        self.version = None
+        last_info = ""
+        with open(self.filepath) as reader:
+            for line in reader:
+                line = line.strip()
+                if self.version is None and line.startswith("bcl2fastq v"):
+                    self.version = line.split(" ")[1]
+                elif self.command is None and "Command-line invocation:" in line:
+                    trace_time, command = line.split(" Command-line invocation: ")
+                    self.command = command
+                    self.start_time = datetime.datetime.strptime(
+                        trace_time.rsplit(" ", 1)[0],
+                        '%Y-%m-%d %H:%M:%S'
+                    )
+                elif line != "":
+                    last_info = line
+        if "Processing completed" in last_info:
+            self.end_time = datetime.datetime.strptime(
+                " ".join(last_info.split(" ", 3)[:2]),
+                '%Y-%m-%d %H:%M:%S'
+            )
+            self.complete = {
+                "nb_errors": int(
+                    re.search(r"(\d+) errors", last_info).group(1)
+                ),
+                "nb_warnings": int(
+                    re.search(r"(\d+) warnings", last_info).group(1)
+                )
+            }
+
+
 class RTAComplete(object):
     """Reader for RTAComplete.txt."""
 
@@ -475,16 +528,19 @@ def getInfFromSeqID(seq_id):
     :return: The sequencer id, run id, flowcell id and position of the sequence on the sequencer flowcell.
     :rtype: dict
     """
-    # Illumina's ID: EAS139:136:FC706VJ:2:2104:15343:197393
-    sequencer_id, run_id, flowcell_id, lane_id, tile_id, x_pos, y_pos = seq_id.split(":")
+    fields = seq_id.split(":")
+    if len(fields) == 7:  # Illumina's ID: EAS139:136:FC706VJ:2:2104:15343:197393
+        fields.append(None)
+    # else  Illumina's ID: EAS139:136:FC706VJ:2:2104:15343:197393:ATGCATA+CTAGC
     return {
-        "sequencer_id": sequencer_id,
-        "run_id": run_id,
-        "flowcell_id": flowcell_id,
-        "lane_id": int(lane_id),
-        "tile_id": int(tile_id),
-        "x_pos": int(x_pos),
-        "y_pos": int(y_pos),
+        "sequencer_id": fields[0],
+        "run_id": fields[1],
+        "flowcell_id": fields[2],
+        "lane_id": int(fields[3]),
+        "tile_id": int(fields[4]),
+        "x_pos": int(fields[5]),
+        "y_pos": int(fields[6]),
+        "umi": fields[7]
     }
 
 
